/* phpio.js
 * by Fabien Cazenave (Kaze)
 *
 * File IO library for ASP/JSP/PHP support (requires kazio.js).
 */

//~ const cgiFILE = /[^\/]*$/;
const cgiTEST = /\~[^\/]*$/;
const cgiREPLACE = /\~([^\/]*$)/;
const cgiWARNING = "*** NOTE THAT THIS FEATURE IS HIGHLY EXPERIMENTAL ***\n\n"
	+ "You should check in the preferences that Nvu does NOT reformat your attributes.\n"
	+ "You should also backup your file first... just in case.\n\n"
	+ "Do you want to edit this file with Nvu?";
	
var phpIO = {

	fileName : function(url) {
		// the 'GetFilename' function does not do the job
		var re = /[^\/]*$/.exec(url);
		//~ return unescape(re[0]);
		return re[0];
	},
	
	fileExt : function(url) {
		var re = /[^\.]*$/.exec(url);
		return re[0];
	},
	
	getDoctype : function(text) {
		// get the doctype
		//~ var doctype = '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">';
		var doctype = '';
		var arr = /^<!DOCTYPE[^>]*/m.exec(text);
		if (arr) {
			doctype = arr[0] + ">";
			//~ text = text.replace(doctype, '');
		}
		
		return doctype;
	},
	
	checkDocument : function(text) {
		// returns true if [text] is a pure HTML document
		var re2 = /^[\s]*<html/i;
		var re3 = /<\/html>[\s]*$/i;
	
		// get the doctype
		//~ var doctype = '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">';
		var doctype = phpIO.getDoctype(text);
		if (doctype != '') {
			text = text.replace(doctype, '');
		}
		
		// check HTML content
		// TODO: check for framesets ?
		return (re2.test(text) && re3.test(text));
	},
	
	checkTemplate : function(text, noWarning) {
		// split file in 3 parts + doctype
		var part1 = '';
		var part2 = '';
		var part3 = '';
		//~ var doctype = '';
		
		// check HTML content
		var arr1 = text.split("<html");
		if (arr1.length <= 1)  // no valid HTML content
			return false; // should open with a text editor
			
		// part1: pre-HTML data (including DOCTYPE, if any)
		if (arr1.length > 1) { // found
			part1 = arr1[0];
			part2 = arr1[1];
		} else
			part2 = arr1[0];
			
		// part2: HTML element
		var arr2 = part2.split("</html>");
		if (arr2.length <= 1)  // no valid HTML content
			return false; // should open with a text editor
		part2 = "<html" + arr2[0] + "</html>";
		
		// part3: post-HTML data
		//~ if (arr2.length > 1) { // found
			part3 = arr2[1];
		//~ }
		
		// get the doctype
		//~ var doctype = '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">';
		//~ var arr = /<!DOCTYPE[^>]*/m.exec(part1);
		//~ if (arr)
			//~ doctype = arr[0] + ">";
		var doctype = phpIO.getDoctype(text);
		
		// user warning
		if (!noWarning) {
			var warning = "This file seems to contain a valid HTML document.\n"
				+ "HandCoder allows to edit the HTML part with Nvu...\n\n"
				+ cgiWARNING;
			if (!ConfirmWithTitle ("HandCoder", warning, "Open with Nvu", "Open with a text editor"))
				return false;
		}
		
		// split the file in two parts
		var dummy = /^[\s]*|[\s]*$/g;
		var htmlPart = doctype + part2;
		var cgiPart = part1.replace(doctype, '').replace(dummy, '') + "\n<html />\n" + part3.replace(dummy, '');
		var parts = new Array(htmlPart, cgiPart);
		return parts;
	},
	
	checkEmbeddable : function(text, noWarning) {
		// look for illegal markup in the file
		var re1 = /<!DOCTYPE[^>]*/m;
		var re2 = /<html|<\/html>|<head|<\/head>|<body|<\/body>/i;
		if (re1.test(text) || re2.test(text))
			return false;
		
		// user warning
		if (!noWarning) {
			var warning = "This file seems to be editable with Nvu...\n\n" + cgiWARNING;
			if (!ConfirmWithTitle ("HandCoder", warning, "Open with Nvu", "Open with a text editor"))
				return false;
		}
		
		// return html file
		var html = "<html><head><title>[fragment]</title></head><body>" + text + "</body></html>";
		return html;
	},
	
	escapePIs : function(text, ext) {
		text = text.replace(/<%/g, '<?php').replace(/%>/g, '?>');
		text = text.replace(/<\?\s/g, '<?php ');
		return text;
	},
	
	unescapePIs : function(text, ext) {
		// ASP or JSP file ?
		if (!(/^php/i.test(ext)))
			text = text.replace(/<\?php/g, '<%').replace(/\?>/g, '%>');
		return text;
	},
	
	Split : function(url, noWarning) {
		if (/^http:/.test(url))
			return url;
		
		// remember to disable attributes reformatting in Nvu prefs
		//~ var fileExt = phpIO.fileExt(url);
		var path = KazIO.urlToPath(url);
		var text = KazIO.read(path);
		
		// is that an HTML document?
		if (phpIO.checkDocument(text)) {
			return url;
		}
		
		// this is not an HTML document, let's start to split the file into editable parts
		var fileNameEscaped = phpIO.fileName(url);
		var fileName = unescape(fileNameEscaped);
		var dirPath = path.replace(fileName, '');
		var parts, html;
		
		// extract PHP nodes, in case there is an "</html>" string in a PI
		//~ var source = text.replace(rePHP, "<??>");
		var source = text;
		const rePHP = /<\?[^\?]*\?>/g;
		var phpNodes = text.match(rePHP);
		if (phpNodes) for (var i = 0; i < phpNodes.length; i++)
			source = source.replace(phpNodes[i], "<?[" + i + "]?>");
		
		// is that a template?
		if (parts = phpIO.checkTemplate(source, noWarning)) {
			var htmlPart = parts[0];
			if (phpNodes) for (i = 0; i < phpNodes.length; i++)
				htmlPart = htmlPart.replace("<?[" + i + "]?>", phpNodes[i]);
			var cgiPart = parts[1];
			if (phpNodes) for (i = 0; i < phpNodes.length; i++)
				cgiPart = cgiPart.replace("<?[" + i + "]?>", phpNodes[i]);
			var htmlFile = '~' + fileName;
			var cgiFile =  '~' + fileName + '.part';
			KazIO.write(dirPath + htmlFile, phpIO.escapePIs(htmlPart));
			KazIO.write(dirPath + cgiFile, cgiPart);
			//~ return url.replace(fileName, htmlFile);
			return url.replace(fileNameEscaped, '~' + fileNameEscaped);	// TODO: use a regexp-based method
		}
		// is that an editable piece of code?
		else if (html = phpIO.checkEmbeddable(text, noWarning)) {
			KazIO.write(dirPath + '~' + fileName, phpIO.escapePIs(html));
			//~ return url.replace(fileName, '~' + fileName);
			return url.replace(fileNameEscaped, '~' + fileNameEscaped);	// TODO: use a regexp-based method
		}
		
		return false;
	},
	
	Merge : function(url) {
		if (!(cgiTEST.test(url)))
			return url;
		
		var fileName = phpIO.fileName(url);
		var fileExt = phpIO.fileExt(url);
		
		// merge the two parts	
		var fileMerge = fileName.replace(/^\~/, '');
		var urlMerge = url.replace(fileName, fileMerge);
		var path = KazIO.urlToPath(urlMerge);
		var part = KazIO.read(KazIO.urlToPath(url + '.part')); // returns 'false' if not found
		var html = KazIO.read(KazIO.urlToPath(url));
		
		// fix some unwanted reformatting
		html = html.replace(/&lt;\?php/g, '<?php');
		html = html.replace(/\?&gt;/g, '?>');
		html = html.replace(/^[\s]*|[\s]*$/g, '');
		html = phpIO.unescapePIs(html, fileExt);
		
		// was that a template?
		if (part) {
			KazIO.write(path, part.replace("<html />", html));
		} 
		// that should be a piece of code
		else {
			test = html.split(/<body>|<\/body>/m);
			//~ KazIO.write(path, test[1]);
			KazIO.write(path, test[1].replace(/^[\s]+/, ''));
			//~ part = test[1];
			//~ while (test = /^\s/.test(part)) {
				//~ part = part.replace(/^\s/g, '');
				//~ part = part.replace(/^\s/m, '');
				//~ part = part.replace(/([\r\n])\s/g, "$1");
			//~ }
			//~ alert(part);
			//~ save(path, part);
		}
		return urlMerge;
	},
	
	Reload : function(url) {
		url = unescape(url);
		var fileName = phpIO.fileName(url);
	
		// check for splitted file
		if (cgiTEST.test(url)) {
			var fileMerge = fileName.replace(/^\~/, '');
			var urlMerge = url.replace(fileName, fileMerge);
			//~ cgiUnlinkTemp(url);
			phpIO.Split(urlMerge, true);
		}
		
		return url;
	},
	
	unlinkTemp : function(url) {
		// check for splitted file
		if (cgiTEST.test(url)) {
			var path = KazIO.urlToPath(url);
			KazIO.unlink(path);
			KazIO.unlink(path + ".part");
		}
		return;
	},
	
	isFragment : function(url) {
		if (!(cgiTEST.test(url)))
			return false;
		var part = KazIO.read(KazIO.urlToPath(url + '.part')); // returns 'false' if not found
		var file = KazIO._newLocalFile(KazIO.urlToPath(url + '.part'));
		return !(file.exists());	
	},
	
	isTemplate : function(url) {
		if (!(cgiTEST.test(url)))
			return false;
		var part = KazIO.read(KazIO.urlToPath(url + '.part')); // returns 'false' if not found
		var file = KazIO._newLocalFile(KazIO.urlToPath(url + '.part'));
		return file.exists();	
	}
	
}